using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using Editroid.UndoRedo;

namespace Editroid
{
	static class Program
	{
		/// <summary>
		/// The main entry points for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);

            mainForm = new frmMain();
			Application.Run(mainForm);
		}

        static frmMain mainForm;
        public static frmMain MainForm { get { return mainForm; } }


        static frmStruct structForm;
        public static bool StructFormIsLoaded { get { return structForm != null && !structForm.IsDisposed; } }
        public static frmStruct StructForm {
            get {
                if (!StructFormIsLoaded)
                    LoadStructForm();

                return structForm;
            }
        }
        public static void LoadStructForm() {
            if (StructFormIsLoaded) return;

            structForm = new frmStruct();
            structForm.Owner = mainForm;
            mainForm.InitializeStructureEditorData(structForm);
        }


        static frmPalette paletteForm;
        public static bool PaletteFormIsLoaded { get { return paletteForm != null && !paletteForm.IsDisposed; } }
        public static frmPalette PaletteForm {
            get {
                if (!PaletteFormIsLoaded)
                    LoadPaletteForm();

                return paletteForm;
            }
        }
        public static void LoadPaletteForm() {
            if (PaletteFormIsLoaded) return;

            paletteForm = new frmPalette();
            paletteForm.Owner = mainForm;
            mainForm.InitializePaletteEditorData(paletteForm);
        }

        public static ActionGenerator Actions { get { return mainForm.Actions; } }
        public static void PerformAction(EditroidAction a) {
            mainForm.PerformAction(a);
        }

        static frmPointers pointerExplererForm;
        public static frmPointers PointerExplorerForm {
            get {
                LoadPointerExplorerForm();
                return pointerExplererForm;
            }
        }
        public static bool PointerExplorerFormIsLoaded { get { return pointerExplererForm != null && !pointerExplererForm.IsDisposed; } }
        public static void LoadPointerExplorerForm() {
            if (PointerExplorerFormIsLoaded) return;

            pointerExplererForm = new frmPointers();
            pointerExplererForm.SetRom(mainForm.GameRom);
            pointerExplererForm.Owner = mainForm;
        }

        private static TestRomPatcher romPatcher = new TestRomPatcher();

        public static TestRomPatcher RomPatcher {
            get { return romPatcher; }
        }

        public static frmAdvancedPalette AdvancedPalForm { get; set; }
        public static void CreateAdvancedPalForm() {
            if(AdvancedPalForm == null || AdvancedPalForm.IsDisposed)
                AdvancedPalForm = new frmAdvancedPalette();
        }
        public static bool AdvancedPalFormIsLoaded { get { return AdvancedPalForm != null && !AdvancedPalForm.IsDisposed; } }

    }
}